import os
import re

from oscars_mgmnt.parser.es.es import ElasticsearchParser


class CLMS_HRVPP_ES_Parser(ElasticsearchParser):

    # Per theme, we define
    #     a dataset (collection) identifier, with substitutions for
    #       projection (EPSG code), resolution, year(s) and short version
    #     a full name for the theme
    dict_theme = {
        'VI': {
            'dataset_identifier': 'copernicus_r_%s_%s_m_hrvpp-vi_p_%s_%s',
            'name': 'Vegetation Indices'
        },
        'ST': {
            'dataset_identifier': 'copernicus_r_%s_%s_m_hrvpp-st_p_%s_%s',
            'name': 'Seasonal Trajectories'
        },
        'VPP': {
            'dataset_identifier': 'copernicus_r_%s_%s_m_hrvpp-vpp_p_%s_%s',
            'name': 'Vegetation Phenology and Productivity parameters'
        }
    }

    # regex to recognize the region name as a S2 tile (syntax TddXXX)
    s2tile_regex=re.compile(r"^T\d{2}")

    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        super().__init__('CLMS_HRVPP.template' if template is None else template, es_source_host, es_source_index)
        # regex used to retrieve the product identifier
        self.regex_identifier = re.compile(r".*/([\w-]*)(?:\.(?:tif|zip)$)")

    def init_dict(self):
        super().init_dict()
        self.dict_info.update(
            {
                # "cloudcover": "properties.cloudcoverpercentage",
                "l1cidentifier": "properties.l1cidentifier"
            }
        )

    def specific_treatment(self, product_file_info, values):
        # Skip the super() method intentionally

        # File name convention:
        # THEME_TIME_[MISSION]_EXTENT-RESOLUTION_VERSION_MODE_FILE-ID
        # example: VI_20180814T100031_S2A_T32TPN-010m_V100_NDVI.tif
        # See URD for more information

        # product identifier (values['id']) should have the form
        # THEME_TIME_[MISSION]_EXTENT-RESOLUTION_VERSION_MODE
        product_base = values['id']
        filename_elem = product_base.split("_")

        mode = None
        mode_long = None
        mission = 'S2'
        if len(filename_elem)==7:
            (theme,nominaltime,mission,spatial,product_version,mode,fileid)=filename_elem
        elif len(filename_elem)==6:
            (theme,nominaltime,mission,spatial,product_version,fileid)=filename_elem

        # Spatial grid information:
        #   geographic area name, projection (EPSG code) and resolution
        #   e.g. E58N22-03035-010m
        spatial_elem = spatial.split('-')
        if self.s2tile_regex.match(spatial_elem[0]):
            # region is a S2 tile
            region, resolution = spatial_elem

            # don't include the starting T in the tileid
            values['tileid'] = region[1:]

            epsg = 'utm-wgs84'
            # conformsto is retained from the ES metadata (S2 tiling grid -> UTM zone)
        else:
            # ETRS89-LAEA = EPSG:3035 projection

            # country code and EPSG (eg at-03035)
            region, epsg, resolution = spatial_elem

            # region is a 2-char ISO code of a country
            values['tileid'] = region

            epsg=epsg.lstrip('0')
            values['conformsto'] = "http://www.opengis.net/def/crs/EPSG/0/3035"

        resolution_value = int(resolution[:-1])
        values['resolution'] = resolution_value

        self.fix_products(product_file_info, epsg, values['resolution'])
        self.fix_geometry(values, simplify=False)

        # Platform "Sentinel-2" is fixed in the template file.
        # Platform's serial id: filename convention allows  S2A, S2B or S2 (= S2A+S2B)
        mission = mission.upper()
        if mission.upper() == "S2":
            values['platformserialid'] = "S2A, S2B"
        else:
            values['platformserialid'] = mission

        values['producttype'] = fileid.upper()

        # Product type / theme
        theme = theme.upper()
        if theme == 'VPP':
            values['productgroupid'] = mode

        # Product version
        #     product_version "Vxyy" like in filename; SDI convention for 'edition' element of ISO XML has xx.yy instead
        #     version_short "vxx_ryy" is used in parent dataset identifier
        #     version_long "version xx revision yy" is used in title
        #  where x = major version, yy = revision / minor version
        product_version=product_version.upper()
        majorversion= int(product_version[1])
        minorversion= product_version[-2:]

        # Parent dataset and collection keep revision = 01
        # to keep V101 and V102 products together
        # version_short = 'v'+ "%02d" % majorversion+'_r'+minorversion
        version_short = 'v'+ "%02d" % majorversion
        version_long = ' '.join(['version', str(majorversion) ,'revision',minorversion])
        if mode:
            mode_long = 'season ' + mode[1]

        # drop the revision in version_long when revision = 00
        if minorversion == '00':
            version_long  = 'version ' + str(majorversion)

        values['productVersion'] = product_version
        year = '2017-now'

        # Title: a readable title, composed of
        #     dataset (collection) title, with substitutions for theme's name, year(s), resolution and long version
        #     tile/area name and nominal time (to identify this product within the collection)
        #     layer name (fileid) is included, as it is used in the file info (per COG file) instead
        #     (VPP parameters only) season 1 or season 2

        if mode:
            values['title'] = '%s %s (raster %s) - %s : %s %s %s %s' % ( self.dict_theme[theme]['name'], year, resolution, version_long, fileid.upper(), region, nominaltime, mode_long)
        else:
            values['title'] = '%s %s (raster %s) - %s : %s %s %s' % ( self.dict_theme[theme]['name'], year, resolution, version_long, fileid.upper(), region, nominaltime)

        if theme == 'VI':
            # example identifier for the parent L1C product: S2B_MSIL1C_20200104T101319_N0208_R022_T33UVS_20200104T121239
            # OPTIONAL - relative orbit nr
            values ['relativeorbitnumber'] = int(values['l1cidentifier'].split('_')[4].replace('R',''))

        if epsg == 'utm-wgs84':
            values['parentidentifier'] = self.dict_theme[theme]['dataset_identifier'] % (epsg, str(resolution_value), year, version_short)
        else: # LAEA collection with mixed resolutions:
            values['parentidentifier'] = self.dict_theme[theme]['dataset_identifier'] % (epsg, 'x', year, version_short)

        # the product ID is retained from convert() and updated by fix_id() call
        uid = values['id']

        return uid

    def fix_id(self, values):
        """ Overwritten - No need to prepend the parent identifier for HR-VPP
        :param values: dictionary containing the values, must include the product identifier and parent identifier
        :type values: dict
        """

    def fix_products(self, fileobject, epsg, resolution):
        """ helper method that moves QA files present in "datas" to "related" and updates file info title
        :param fileobject: the files associated to a product
        :param epsg: EPSG code
        :param resolution: image resolution
        :type fileobject: a FileObject
        """

        for info in list(fileobject.datas):
            file_base = os.path.splitext(os.path.basename(info.name))[0]
            filename_elem = file_base.split("_")

            if epsg == 'utm-wgs84':
                info.title = filename_elem[-1].upper()
            else:
                info.title = filename_elem[-1].upper() + '_' + str(resolution) + 'M'

            # add human readable name of the fileid (layer) to file info or leave this up to front-end?
            if "QFLAG" in info.title or "QFLAG2" in info.title:
                info.category = "QUALITY"
                # fileobject.datas.remove(info)
                # fileobject.related.append(info)
