import re

from oscars_mgmnt.parser.es.es import ElasticsearchParser


class Sentinel1_ES_Parser(ElasticsearchParser):

    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        super().__init__('CGS_S1.template' if template is None else template, es_source_host, es_source_index)

    def init_dict(self):
        super().init_dict()
        self.dict_info.update(
            {
                "polarisationmode": "properties.polarisationmode",
                "srs": "properties.srs"
            }
        )

    def specific_treatment(self, product_file_info, values):
        uid = super().specific_treatment(product_file_info, values)
        self.fix_geometry(values, simplify=False)

        values['platformserialid'] = uid.split("_")[0]

        if "SIGMA0" in values["producttype"]:
            values["producttype"] = "SIGMA0"
            values['parentidentifier'] = "urn:eop:VITO:CGS_S1_GRD_SIGMA0_L1"
        elif "COHERENCE" in values["producttype"]:
            values["producttype"] = "COHERENCE"
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S1_SLC_COHERENCE_V1"
            values['nominaldatetime'] = self.format_uid_date(uid.split("_")[4])

            # don't include platformserialid for coherence products
            values.pop('platformserialid', None)
        elif values["producttype"] in ["GRD", "SLC"]:
            values['parentidentifier'] = f"urn:eop:VITO:CGS_S1_{values['producttype']}_L1"
            values['conformsto'] = "http://www.opengis.net/def/crs/EPSG/0/4326"

        if "srs" in values:
            values['referenceSystemIdentifier'] = values['srs']
            values['conformsto'] = f"https://www.opengis.net/def/crs/EPSG/0/{values['srs'].split(':')[1]}"

        polarisations = {
            "SH": {"mode": "S", "channels": "HH"},
            "SV": {"mode": "S", "channels": "VV"},
            "DH": {"mode": "D", "channels": "HH, HV"},
            "DV": {"mode": "D", "channels": "VV, VH"},
            "HH": {"mode": "D", "channels": "HH"},
            "HV": {"mode": "D", "channels": "HV"},
            "VV": {"mode": "D", "channels": "VV"},
            "VH": {"mode": "D", "channels": "VH"}
        }

        polarisation = [v for v in polarisations.values() if all(x in v['channels'] for x in values['polarisationmode'].split())].pop()

        values["polarisationmode"] = polarisation["mode"]
        values["polarisationchannels"] = polarisation["channels"]

        wms_layer = "CGS_S1_COHERENCE" if "COHERENCE" in values["producttype"] else "CGS_S1_GRD_SIGMA0"
        self.add_wms_quicklook(wms_layer, product_file_info, values)

        return uid

    def fix_products(self, fileobject):
        grd_pattern = re.compile(r".*S1._.._GRDH_.*.zip")
        slc_pattern = re.compile(r".*S1._.._SLC__.*.zip")
        sigma0_pattern = re.compile(r".*S1._.._GRDH_SIGMA0_.*_V\d{3}_(angle|VV|VH).tif")
        coherence_pattern = re.compile(r".*S1._S1._Coherence_.*_V\d{3}_(VV|VH).tif")
        SAFE_pattern = re.compile(r'.*\.SAFE/(.*)')

        for info in list(fileobject.datas):
            if sigma0_pattern.match(info.name):
                info.title = sigma0_pattern.search(info.name).group(1)
            elif coherence_pattern.match(info.name):
                info.title = coherence_pattern.search(info.name).group(1)
            elif grd_pattern.match(info.name):
                info.title = "GRD"
            elif slc_pattern.match(info.name):
                info.title = "SLC"

        for info in list(fileobject.metadatas):
            if SAFE_pattern.match(info.name):
                fileobject.metadatas.remove(info)
                fileobject.related.append(info)

        for info in fileobject.related + fileobject.datas:
            if SAFE_pattern.match(info.name):
                info.title = SAFE_pattern.search(info.name).group(1)
